/*****************************************************************************/
/* AUTOR: Juan Fco Rodriguez Hervella                                        */
/* FECHA: 2-Marzo-2001                                                       */
/* VERSION: 1.0                                                              */
/* OBJETIVO:                                                                 */
/*  Realizar ping6 desde maquinas con soporte IPv4 solamente, a maquinas     */
/*  con soporte de tuneles 6to4. Me he basado en la implementacion de ping   */
/*  utilizada en UNIX Network Progamming - Networking APIs: Sockets and XTI, */
/*  de R. Stevens.                                                           */
/*****************************************************************************/
								    
#include <stdio.h>

#include "ping6to4.h"  /* structs for ip6/icmp6 headers, and constanst */
#include "utils.h"    /* inet_pton() inet_ntop(), and auxiliary functions */


/*******************************************************************/
/*              GLOBAL VARS.                                       */
/*******************************************************************/
								    					    
char recvbuf[BUFSIZE];
char sendbuf[BUFSIZE];
char auxbuf[BUFSIZE];
int  nsent;        /* add 1 for each sento() */
int  nreceived;    /* add 1 for each recvfrom() */
char *host;        /* string with the IPv6 address */
int sockfd;        /* raw socket IPv4 */
int verbose;       
int pid;           /* pid used to start various ping6-4 without conflict */
int datalen = 56;  /* data that goes with ICMP echo request */

float tmin = 999999999;
float tmax = 0;
float tsum = 0;	       /* sum of all times, for doing average */


struct sockaddr_in  origen_ip;
struct sockaddr_in  destino_ip;

struct sockaddr_in6 origen_ip6;
struct sockaddr_in6 destino_ip6;


/*******************************************************************/
/*              FUNCTION PROTOTYPES                                */					    
/*******************************************************************/

void sig_alrm( int signal );
void finish( int signal );
void send_packet();
void process_packet( char *ptr, int len, struct timeval *tvrecv );
void readloop();

/********************************************************************/
/*                    CODE                                          */
/********************************************************************/
								    
void finish( int signal )
{
  putchar('\n');
  fflush(stdout);
  printf("\n----%s PING6-4 Statistics----\n", host );
  printf("%d packets transmitted, ", nsent );
  printf("%d packets received, ", nreceived );  
  printf("%d%% packet loss", 
	 (int) (((nsent-nreceived)*100) / nsent));
  printf("\n");
  if( nreceived != 0 )
    printf("round-trip (ms)  min/avg/max = %.3f/%.3f/%.3f\n\n", tmin,
	   tsum / nreceived, tmax );
  else
    printf("\n");

  fflush(stdout);
  exit(0);
}

  
void sig_alrm( int signal )
{
  send_packet();

  alarm(1);
  return;  /* probably interrupts recvfrom() */
}


void send_packet()
{
  int len;
  struct ip6_hdr *ip6;
  struct icmp6_rr *icmp6;
  struct ip6_pseudo_hdr *pseudo_ip6; 

  ip6= (struct ip6_hdr *)sendbuf;
  ip6->ip6_ctl.ip6_hdrctl.ip6_flow = 0;
  ip6->ip6_ctl.ip6_hdrctl.ip6_plen = htons( sizeof(struct icmp6_rr) + datalen );
  ip6->ip6_ctl.ip6_hdrctl.ip6_nxt = IPPROTO_ICMPV6;
  ip6->ip6_ctl.ip6_hdrctl.ip6_hlim = 255;
  ip6->ip6_ctl.ip6_vtc = 6 << 4;
  memcpy( &ip6->ip6_src, &origen_ip6.sin6_addr, 16 );
  memcpy( &ip6->ip6_dst, &destino_ip6.sin6_addr, 16 );

  icmp6= (struct icmp6_rr *)(sendbuf + sizeof(struct ip6_hdr));
  icmp6->icmp6_type = ICMP6_ECHO_REQUEST;
  icmp6->icmp6_code = 0;  
  icmp6->icmp6_id = pid;
  icmp6->icmp6_seq = nsent++;
  
  gettimeofday( (struct timeval *)(icmp6 + 1), NULL );

  len = sizeof(struct ip6_hdr) + sizeof(struct icmp6_rr) + datalen;

  icmp6->icmp6_cksum = 0;

  memcpy( auxbuf + sizeof(struct ip6_pseudo_hdr), icmp6, sizeof(struct icmp6_rr) + datalen );
  pseudo_ip6= (struct ip6_pseudo_hdr *)auxbuf;

  pseudo_ip6->ip6_src= ip6->ip6_src;
  pseudo_ip6->ip6_dst= ip6->ip6_dst;
  pseudo_ip6->ip6_plen= htonl( sizeof(struct icmp6_rr) + datalen );
  pseudo_ip6->ip6_nxt = htonl( IPPROTO_ICMPV6 );

  icmp6->icmp6_cksum = in_cksum( (u_short *)pseudo_ip6, sizeof(struct ip6_pseudo_hdr) + 
				                        sizeof(struct icmp6_rr) + 
				                        datalen );

  if( sendto( sockfd, sendbuf, len, 0, (struct sockaddr *)&destino_ip, sizeof(struct sockaddr_in) ) != len )
  {
    perror("error in sendto():");
    exit(-1);     
  } 
}

void process_packet( char *ptr, int len, struct timeval *tvrecv )
{
  struct ip *ip;
  struct ip6_hdr *ip6; 
  struct icmp6_rr *icmp6;
  int hlen_ip, hlen_ip6, icmp6len;
  struct timeval *tvsend;
  double rtt;
  char aux[INET6_ADDRSTRLEN];
  
  ip= (struct ip *)ptr; /* start of IP header */
  hlen_ip = ip->ip_hl << 2; /* length of IP header */
  
  ip6= (struct ip6_hdr *)(ptr + hlen_ip);  /* start of IPv6 header */
  hlen_ip6= sizeof(struct ip6_hdr);
  if( ip6->ip6_ctl.ip6_hdrctl.ip6_nxt != IPPROTO_ICMPV6 )
  {
    //printf("next header not IPPROTO_ICMPV6: %d\n", ip6->ip6_ctl.ip6_hdrctl.ip6_nxt );
    return;  // it is another encapsulate packet.
  }

  icmp6 = (struct icmp6_rr *)(ptr + hlen_ip + hlen_ip6);
  if( (icmp6len = len - hlen_ip - hlen_ip6 ) < 8)
  {
    printf("icmp6len (%d) < 8", icmp6len);
    exit(-1);
  }

  if( icmp6->icmp6_type == ICMP6_ECHO_REPLY )
  {
    if( icmp6->icmp6_id != pid )
      return;  /* not a response to our ECHO_REQUEST */
    if(icmp6len < 16)
    {
      printf("icmp6len (%d) < 16", icmp6len );
      exit(-1);
    }

    tvsend = (struct timeval *)(icmp6 + 1);
    tv_sub(tvrecv, tvsend);
    rtt= tvrecv->tv_sec * 1000.0 + tvrecv->tv_usec / 1000.0;
   
    tsum += rtt;
    if( rtt < tmin )
      tmin = rtt;
    if( rtt > tmax )
      tmax = rtt;

    printf("%d bytes from %s: seq=%u, hlim=%d, rtt=%.3f ms\n",
	   icmp6len, inet6_ntop( &ip6->ip6_src, aux, INET6_ADDRSTRLEN ), icmp6->icmp6_seq,
	   ip6->ip6_ctl.ip6_hdrctl.ip6_hlim, rtt );
  }
  else
  if(verbose)
  {
    printf(" %d bytes from %s: type = %d, code = %d\n",
	   icmp6len, inet6_ntop( &ip6->ip6_src, aux, INET6_ADDRSTRLEN ), icmp6->icmp6_type,
	   icmp6->icmp6_code );
  }

  nreceived++;
 
}

void readloop()
{
  int size;
  int n;
  struct timeval tval;
  int len_origen_ip;
 
  if( (sockfd= socket(PF_INET, SOCK_RAW, IPPROTO_IPV6 )) < 0 )
  {
    perror("error in socket():");
    exit(-1);      
  }
  setuid(getuid()); /* don't need special permissions any more */

  size= 60*1024;
  if( setsockopt( sockfd, SOL_SOCKET, SO_RCVBUF, &size, sizeof(size) ) < 0 )
  {
    perror("error in setsockopt():");
    exit(-1);
  }

  /* assign source IPv4 address to the packet */ 
  if( bind( sockfd, (struct sockaddr *)&origen_ip, sizeof(struct sockaddr_in) ) < 0 )
  {
    perror("error in bind:");
    exit(-1);
  }
  
  sig_alrm(SIGALRM); /* send first packet */

  for( ; ; )
  {
    n = recvfrom( sockfd, recvbuf, sizeof(recvbuf), 0, 
		  (struct sockaddr *)&origen_ip, &len_origen_ip );
    if( n < 0 )
    {
      if( errno == EINTR)
	continue;
      else
      {
	printf("recvfrom error\n");
	exit(-1);	
      }
    }   

    gettimeofday( &tval, NULL );
    process_packet( recvbuf, n, &tval ); // increments nreceived if all is ok.  
  }
}

/********************************************************************/
/*                    FUNCTION MAIN                                 */
/********************************************************************/

int main( int argc, char **argv)
{
  int c;  
  extern int optind, opterr;
  extern char *optarg;  

  optind = 1; 
  opterr = 0;
  while( (c=getopt(argc, argv, "s:v")) != -1 )
  {
    switch(c)
    {
    case 'v':
      verbose++;
      break;
    case 's':
      origen_ip.sin_family = AF_INET;
      if( (origen_ip.sin_addr.s_addr= inet_addr( optarg )) == -1 ) 
      {
	printf("error in inet_addr(): bad address\n");
	exit(-1);
      }
      break;
    case '?':
      printf("unrecognized option: %c\n", c);
      exit(-1);
    }
  }

  if(optind != argc - 1 )
  {
    printf("usage: ping6-4 [-v] -s <origen_IPv4> <destino_IPv6 (2002:)\n>");
    exit(-1);
  }
    
  host= argv[optind];

  pid= getpid();

  signal(SIGALRM, sig_alrm);
  signal( SIGINT, finish );

  inet6_pton( host, &destino_ip6.sin6_addr );
  
  /* sets the packet source/dest address acording to RFC 3056 (tunnel 6to4) */
  destino_ip.sin_family = AF_INET;
  destino_ip.sin_addr.s_addr= get_destino_ip( &destino_ip6 );
  set_origen_ip6( &origen_ip6, &origen_ip );
 
  printf("PING6-4 %s : %d data bytes\n", host, datalen );

  readloop(); 
  
  exit(0);
}
