#ifndef ping64h
#define ping64h

#include <sys/time.h>  
#include <unistd.h> 
#include <signal.h>  /* for signaling */
#include <errno.h>  /* for errno. */
#include <sys/socket.h> /* for setsockopt() */

struct in_addr
{
  u_int32_t s_addr;
};

struct in6_addr
{
  union
  {
    u_int8_t         u6_addr8[16];
    u_int16_t        u6_addr16[8];
    u_int32_t        u6_addr32[4];
#if ULONG_MAX > 0xffffffff
    u_int64_t        u6_addr64[2];
#endif
  } in6_u;    
#define s6_addr			in6_u.u6_addr8
#define s6_addr16		in6_u.u6_addr16
#define s6_addr32		in6_u.u6_addr32
#define s6_addr64		in6_u.u6_addr64          
};            

#define __SOCKADDR_COMMON(sa_prefix) \
  sa_family_t sa_prefix##family
                                
#define __SOCKADDR_COMMON_SIZE  (sizeof (unsigned short int))     


struct sockaddr_in
{
  __SOCKADDR_COMMON (sin_);
  u_int16_t sin_port;			/* Port number.  */
  struct in_addr sin_addr;		/* Internet address.  */

  /* Pad to size of `struct sockaddr'.  */
  unsigned char sin_zero[sizeof (struct sockaddr) -
			__SOCKADDR_COMMON_SIZE -
			sizeof (u_int16_t) -
			sizeof (struct in_addr)];
};

struct sockaddr_in6
{
  __SOCKADDR_COMMON (sin6_);
  u_int16_t sin6_port;		/* Transport layer port # */
  u_int32_t sin6_flowinfo;	/* IPv6 flow information */
  struct in6_addr sin6_addr;	/* IPv6 address */
};


struct ip
{
#if BYTE_ORDER == LITTLE_ENDIAN
  unsigned int ip_hl:4;		/* header length */
  unsigned int ip_v:4;		/* version */
#endif
#if BYTE_ORDER == BIG_ENDIAN
  unsigned int ip_v:4;		/* version */
  unsigned int ip_hl:4;		/* header length */
#endif
  u_int8_t ip_tos;			/* type of service */
  u_short ip_len;			/* total length */
  u_short ip_id;			/* identification */
  u_short ip_off;			/* fragment offset field */
#define	IP_RF 0x8000			/* reserved fragment flag */
#define	IP_DF 0x4000			/* dont fragment flag */
#define	IP_MF 0x2000			/* more fragments flag */
#define	IP_OFFMASK 0x1fff		/* mask for fragmenting bits */
  u_int8_t ip_ttl;			/* time to live */
  u_int8_t ip_p;			/* protocol */
  u_short ip_sum;			/* checksum */
  struct in_addr ip_src, ip_dst;	/* source and dest address */
};

struct icmp6_rr  /* RFC 2463 */
{
  u_int8_t     icmp6_type;   /* type field */
  u_int8_t     icmp6_code;   /* code field */
  u_int16_t    icmp6_cksum;  /* checksum field */
  u_int16_t    icmp6_id;     /* identifier */
  u_int16_t    icmp6_seq;    /* # of sequence */
}; 


struct ip6_hdr 
{
  union 
  {
    struct ip6_hdrctl 
    {
      u_int32_t ip6_flow;   /* 24 bits of flow-ID */
      u_int16_t ip6_plen;   /* payload length */
      u_int8_t  ip6_nxt;    /* next header */
      u_int8_t  ip6_hlim;   /* hop limit */
    } ip6_hdrctl;

    u_int8_t ip6_vtc;       /* 4 bits version, 4 bits priority */
  } ip6_ctl;

  struct in6_addr ip6_src;      /* source address */
  struct in6_addr ip6_dst;      /* destination address */
};                                    

struct ip6_pseudo_hdr  /* for calculate the ICMPv6 checksum */
{
  struct in6_addr ip6_src;
  struct in6_addr ip6_dst;
  u_int32_t       ip6_plen;
  u_int32_t       ip6_nxt;
};


#define IPPROTO_IPV6 41 /* protocol type for encapsulation of IPv6 inside of IPv4 packets */
#define INET6_ADDRSTRLEN 46
#define IPPROTO_ICMPV6 58
#define ICMP6_ECHO_REQUEST  128
#define ICMP6_ECHO_REPLY    129
#define BUFSIZE 1500



#endif
