#include "utils.h"

/*
 *			I N _ C K S U M
 *
 * Checksum routine for Internet Protocol family headers (C Version)
 *
 */
u_short in_cksum( u_short *addr, int len)
{
  register int nleft = len;
  register u_short *w = addr;
  register u_short answer;
  register int sum = 0;

  /*
   *  Our algorithm is simple, using a 32 bit accumulator (sum),
   *  we add sequential 16 bit words to it, and at the end, fold
   *  back all the carry bits from the top 16 bits into the lower
   *  16 bits.
   */
  while( nleft > 1 )  
  {
    sum += *w++;
    nleft -= 2;
  }

  /* mop up an odd byte, if necessary */
  if( nleft == 1 ) 
  {
    u_short	u = 0;

    *(u_char *)(&u) = *(u_char *)w ;
    sum += u;
  }

  /*
   * add back carry outs from top 16 bits to low 16 bits
   */
  sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
  sum += (sum >> 16);			/* add carry */
  answer = ~sum;				/* truncate to 16 bits */
  
  return (answer);
}

void tv_sub( struct timeval *out, struct timeval *in)
{
  if((out->tv_usec -= in->tv_usec) < 0 )
  {
    --out->tv_sec;
    out->tv_usec += 1000000;
  }

  out->tv_sec -= in->tv_sec;
}

short to_decimal( char car1, char car2 )
{   
  u_int8_t num;

  switch( car1 )
  {
  case '0':
    num = 0;
    break;
  case '1':
    num = 1;
    break;
  case '2':
    num = 2;
    break;
  case '3':
    num = 3;
    break;
  case '4':
    num = 4;
    break;
  case '5':
    num = 5;
    break;
  case '6':
    num = 6;
    break;
  case '7':
    num = 7;
    break;
  case '8':
    num = 8;
    break;
  case '9':
    num = 9;
    break;
  case 'a':
  case 'A':
    num = 10;
    break;
  case 'b':
  case 'B':
    num = 11;
    break;
  case 'c':
  case 'C':
    num = 12;
    break;
  case 'd':
  case 'D':
    num = 13;
    break;
  case 'e':
  case 'E':
    num = 14;
    break;
  case 'f':
  case 'F':
    num = 15;
    break; 
  }

  num = num << 4;

  switch( car2 )
  {
  case '0':
    num += 0;
    break;
  case '1':
    num += 1;
    break;
  case '2':
    num += 2;
    break;
  case '3':
    num += 3;
    break;
  case '4':
    num += 4;
    break;
  case '5':
    num += 5;
    break;
  case '6':
    num += 6;
    break;
  case '7':
    num += 7;
    break;
  case '8':
    num += 8;
    break;
  case '9':
    num += 9;
    break;
  case 'a':
  case 'A':
    num += 10;
    break;
  case 'b':
  case 'B':
    num += 11;
    break;
  case 'c':
  case 'C':
    num += 12;
    break;
  case 'd':
  case 'D':
    num += 13;
    break;
  case 'e':
  case 'E':
    num += 14;
    break;
  case 'f':
  case 'F':
    num += 15;
    break; 
  }

  return num;
}

char *inet6_ntop( struct in6_addr *addr, char *host, int size )
{     
        static char digits[] = "0123456789abcdef";
	int i;
	char *cp;
	u_short *a = (u_short *)addr;
	u_char *d;
	int dcolon = 0;
       
	if( size < INET6_ADDRSTRLEN )
	{
	  printf("size too short in ine6_ntop()\n");
	  exit(-1);
	}
     
	cp = host;

	for (i = 0; i < 8; i++) {
		if (dcolon == 1) {
			if (*a == 0) {
				if (i == 7)
					*cp++ = ':';
				a++;
				continue;
			} else
				dcolon = 2;
		}
		if (*a == 0) {
			if (dcolon == 0 && *(a + 1) == 0) {
				if (i == 0)
					*cp++ = ':';
				*cp++ = ':';
				dcolon = 1;
			} else {
				*cp++ = '0';
				*cp++ = ':';
			}
			a++;
			continue;
		}

		d = (u_char *)a;
		if( *d != 0 )
		{
		  if( (*d >> 4) != 0 )
		    *cp++ = digits[*d >> 4];
		  *cp++ = digits[*d++ & 0xf];
		  *cp++ = digits[*d >> 4];
		  *cp++ = digits[*d & 0xf];
		}
		else
		{
		  d++;
		  if( (*d >> 4) != 0 )
		    *cp++ = digits[*d >> 4];
		  *cp++ = digits[*d & 0xf];
		}		
		*cp++ = ':';
		a++;
	}

	*--cp = 0;       
	return host;
}


int inet6_pton( char *host, struct in6_addr *addr )
{
  int i, j, k, p, r;
  int zero, num_hex;
  char aux[INET6_ADDRSTRLEN]; 
  char acum_str[5];
  int no_more= 0;

  if( (host[0] == ':') && (host[1] != ':') )
    return -1; /* start with :  is not permmited */

  /* count the # of characters hexadecimal */ 
  num_hex= 0;
  i= 0;
  while( (host[i] != '\0') && (i<=INET6_ADDRSTRLEN) )
  { 
    if( (host[i] == ':') && (host[i+1] != ':') )
      num_hex = num_hex+4;    

    i++;   
  }

  if( ! ((host[i-2] == ':')&&(host[i-1] == ':')) )
    num_hex= num_hex + 4;

  if( (host[0] == ':')&&(host[1] == ':') )
    num_hex = num_hex - 4;

  if( i>INET6_ADDRSTRLEN )
    return -1; /* overflow */

  /* calculate the # of zeroes to fill the gap between :: */  
  zero = 32 - num_hex;
  if( zero % 4 != 0 )
    return -1;  /* the # of zeroes between :: must be multiplier of four */  

  /* convert the string to long format */
  i= 0;
  k= 0;
  p= 0;
  while( host[i] != '\0' )
  {
    if( (host[i] == ':') && (host[i+1] == ':') )
    {
      if( host[i+2] == ':' )
	return -1;  /* ::: is an invalid format string */
      if( no_more )
	return -1; /* :: must appear only one time */

      if( p != 0 )
      {
	acum_str[p]= '\0';
	for(r=0; r<(4-strlen(acum_str)); r++)
	{
	  aux[k]='0';
	  k++;
	}
	sprintf( aux+k, "%s:", acum_str );
	k= k+strlen(acum_str)+1;
	p= 0;
      }

      for(j=0; j<zero ;j++)
      {
	if( (j % 4 == 0) && (j != 0) )	
	{
	  aux[k]= ':';
	  k++;
	}
	
	aux[k]= '0';
	k++;       
      }
     
      aux[k]= ':';
      k++;
      i++;
      no_more= 1;
    }
    else
    {
      if( host[i] == ':' )
      {
	acum_str[p]= '\0';
	for(r=0; r<(4-strlen(acum_str)); r++)
	{
	  aux[k]='0';
	  k++;
	}
	sprintf( aux+k, "%s:", acum_str );
	k= k+strlen(acum_str)+1;
	p= 0;
      }
      else
      {
	acum_str[p]= host[i];
	p++;
      }
      
    }

    i++;    
  }
     
  if(p != 0)
  {
    acum_str[p]= '\0';
    for(r=0; r<(4-strlen(acum_str)); r++)
    {
      aux[k]='0';
      k++;
    }
    sprintf( aux+k, "%s", acum_str );
    k= k+strlen(acum_str);
    aux[k]= '\0';
  }
  else
    aux[k-1]= '\0';

  /* convert the long formatted address characters to int and save in network
     byte order in the in6_addr struct */
  i= 0;
  j= 0;
  while( aux[i] != '\0' )
  {
    if( aux[i] == ':' )
      i++;

    addr->s6_addr[j]= to_decimal( aux[i], aux[i+1] );
    j++;
    
    i= i+2;   
  }

  return 0;
}

u_int32_t get_destino_ip( struct sockaddr_in6 *destino_ip6 )
{ 
  u_int32_t num;

  memcpy( (u_int16_t *)&num, destino_ip6->sin6_addr.s6_addr16+1, 2);
  memcpy( (u_int16_t *)((u_int16_t *)&num + 1), destino_ip6->sin6_addr.s6_addr16+2, 2 );
  
  return num;
}

void set_origen_ip6(struct sockaddr_in6 *origen_ip6, struct sockaddr_in *origen_ip)
{  
  u_int16_t *aux;
  char dir[INET6_ADDRSTRLEN]; 
  
  aux= (u_int16_t *)&origen_ip->sin_addr.s_addr;

  strcpy( dir, "2002:" ); 
  sprintf( dir+5, "%04x:", ntohs( aux[0] ) );
  sprintf( dir+5+5, "%04x::1", ntohs( aux[1] ) );

  inet6_pton( dir, &origen_ip6->sin6_addr );
}

